//
// duo-touch-sensor.c
//
// step response loading hello-world
//    9600 baud FTDI interface
//
// Neil Gershenfeld
// 10/27/10
//
// (c) Massachusetts Institute of Technology 2010
// Permission granted for experimental and personal use;
// license for commercial sale available from MIT.
//
// Changes added by Linda Wanders
// FabAcademy2014 - final project development 
// 4/28/14
//

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set

#define bit_delay_time 100 // bit delay for 9600 with overhead
#define bit_delay() _delay_us(bit_delay_time) // RS232 bit delay
#define half_bit_delay() _delay_us(bit_delay_time/2) // RS232 half bit delay
#define char_delay() _delay_ms(10) // char delay

#define charge_delay_1() _delay_us(1) // charge delay 1
#define charge_delay_2() _delay_us(10) // charge delay 2
#define charge_delay_3() _delay_us(100) // charge delay 3
#define settle_delay() _delay_us(100) // settle delay

#define serial_port PORTB
#define serial_direction DDRB
#define serial_pin_out (1<<PB2)
#define serial_pin_in (1<<PB0)
#define serial_pins PINB

#define serial_interrupt (1 << PCIE)
#define serial_interrupt_pin (1 << PCINT0)

#define charge_port PORTB
#define charge_direction DDRB
#define charge_pin (1 << PB1)

#define node_id '1'

int16_t measure_interval_1 ();
int16_t measure_interval_2 ();
int16_t measure_interval_3 ();

int16_t adc0_avg1, adc0_avg2, adc0_avg3; //declare varialble
int16_t adc1_avg1, adc1_avg2, adc1_avg3;

volatile char chr; //globale variabeles

void send_result ();
void send_framing ();


void get_char(volatile unsigned char *pins, unsigned char pin, volatile char *rxbyte) {
   //
   // read character into rxbyte on pins pin
   //    assumes line driver (inverts bits)
   //
   *rxbyte = 0;
   while (pin_test(*pins,pin))
      //
      // wait for start bit
      //
      ;
   //
   // delay to middle of first data bit
   //
   half_bit_delay();
   bit_delay();
   //
   // unrolled loop to read data bits
   //
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 0);
   else
      *rxbyte |= (0 << 0);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 1);
   else
      *rxbyte |= (0 << 1);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 2);
   else
      *rxbyte |= (0 << 2);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 3);
   else
      *rxbyte |= (0 << 3);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 4);
   else
      *rxbyte |= (0 << 4);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 5);
   else
      *rxbyte |= (0 << 5);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 6);
   else
      *rxbyte |= (0 << 6);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 7);
   else
      *rxbyte |= (0 << 7);
   //
   // wait for stop bit
   //
   bit_delay();
   half_bit_delay();
   }

void put_char(volatile unsigned char *port, unsigned char pin, char txchar) {
   //
   // send character in txchar on port pin
   //    assumes line driver (inverts bits)
   //
   // start bit
   //
   clear(*port,pin);
   bit_delay();
   //
   // unrolled loop to write data bits
   //
   if bit_test(txchar,0)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,1)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,2)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,3)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,4)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,5)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,6)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,7)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   //
   // stop bit
   //
   set(*port,pin);
   bit_delay();
   //
   // char delay
   //
   bit_delay();
   }

ISR(PCINT0_vect) {
   //
   // pin change interrupt handler
   //
       get_char(&serial_pins, serial_pin_in, &chr);
      //put_char(&serial_port, serial_pin_out, chr);
	  GIFR = (1<<PCIF); //clear interrupt flag which was set by the pin changing during the interrupt
   }
   
int main(void) {
   //
   // main
   //
   // set clock divider to /1
   //
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
   //
   // initialize output pins
   //
   set(serial_port, serial_pin_out);
   set(serial_port, serial_pin_in);
   output(serial_direction, serial_pin_out);
   clear(charge_port, charge_pin);
   output(charge_direction, charge_pin);
   //
   // setup pin change interrupt on input pin
   //
   set(GIMSK, serial_interrupt);
   set(PCMSK, serial_interrupt_pin);
   //
   // initialize A/D/C (analog to digital convertor)
   //
   ADCSRA = (1 << ADEN) // enable 
      | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0); // prescaler /128
   //
   //
   //
   chr=0; // initialize variable
   sei(); // set enable interupts 
   //
   // main loop
   //
   while (1) {
      //put_char(&serial_port, serial_pin_out, chr);
	  //set adc to PB4
	  ADMUX = (0 << REFS2) | (0 << REFS1) | (0 << REFS0) // Vcc ref
      | (0 << ADLAR) // right adjust
      | (0 << MUX3) | (0 << MUX2) | (1 << MUX1) | (0 << MUX0); // PB4 (sense) 
		
	  //
	  // manier om te bereken zonder de geschiedenis bij te houden
	  // nieuwe gemidd waarde = (3*oude gemid. waarde + nieuwe waarde) / 4
	  //
	  
	  
	  
	  int16_t lalala = measure_interval_1(); // the name (lalala) is(=) equal to 'result of function' (interval)

	  if (chr==0) {
	    adc0_avg1 = (3*adc0_avg1 + lalala) / 4; // when nothing happens, calculate and continue
	  }
	  else {
	    if(chr==node_id) {
	      send_result ();
	    }
	  chr=0; //back to staring value (clear)
	  }
	

	  lalala = measure_interval_2();
	  if (chr==0) {
	    adc0_avg2 = (3*adc0_avg2 + lalala) / 4; 
	  }
	  else {
	    if(chr==node_id) {
	      send_result ();
	    }
	  chr=0; 
	  }
		
		
	  lalala = measure_interval_3();
	  if (chr==0) {
	    adc0_avg3 = (3*adc0_avg3 + lalala) / 4; 
	  }
	  else {
	    if(chr==node_id) {
	      send_result ();
	    }
	  chr=0; 
	  }
	  
	  
	  
	  // set adc to PB3
	  ADMUX = (0 << REFS2) | (0 << REFS1) | (0 << REFS0) // Vcc ref
      | (0 << ADLAR) // right adjust
      | (0 << MUX3) | (0 << MUX2) | (1 << MUX1) | (1 << MUX0); // PB3 (sense)
	 
	 
	 
	  lalala = measure_interval_1();
	  if (chr==0) {
	    adc1_avg1 = (3*adc1_avg1 + lalala) / 4; 
	  } else {
	    if(chr==node_id) {
	      send_result ();
	    }
	    chr=0; 
	  }
	  
	  
	  lalala = measure_interval_2();
	  if (chr==0) {
	    adc1_avg2 = (3*adc1_avg2 + lalala) / 4; 
	  }
	  else {
	    if(chr==node_id) {
	      send_result ();
	    }
	  chr=0; 
	  }
	  
	  
	  lalala = measure_interval_3();
	  if (chr==0) {
	    adc1_avg3 = (3*adc1_avg3 + lalala) / 4; 
	  }
	  else {
	    if(chr==node_id) {
	      send_result ();
	    }
	  chr=0; 
	  }	 
	  
	  // send_framing ();
	  // send_result ();
	  
      }
   }
   
 int16_t measure_interval_1 () { 
   
   uint16_t up, down; // uint16_t is a type of variable    
	  //
      // settle, charge, and wait 1
      //
      settle_delay();
      set(charge_port, charge_pin);
      charge_delay_1();
      //
      // initiate conversion
      //
      ADCSRA |= (1 << ADSC);
      //
      // wait for completion
      //
      while (ADCSRA & (1 << ADSC))
         ;
      //
      // save result
      //
      up = ADC;
      //
      // settle, discharge, and wait 1
      //
      settle_delay();
      clear(charge_port, charge_pin);
      charge_delay_1();
      //
      // initiate conversion
      //
      ADCSRA |= (1 << ADSC);
      //
      // wait for completion
      //
      while (ADCSRA & (1 << ADSC))
         ;
      //
      // save result
      //
      down=ADC;

	return ( (up + (1023 - down))/2);
	// example code : value1 = (up_value1 + (1023 - down_value1))/2.0
}
 
 int16_t measure_interval_2 () { 
   
   uint16_t up, down;

	  //
	  // settle, charge, and wait 2
      //
      settle_delay();
      set(charge_port, charge_pin);
      charge_delay_2();
      //
      // initiate conversion
      //
      ADCSRA |= (1 << ADSC);
      //
      // wait for completion
      //
      while (ADCSRA & (1 << ADSC))
         ;
      //
      // save result
      //
      up = ADC;
      //
      // settle, discharge, and wait 2
      //
      settle_delay();
      clear(charge_port, charge_pin);
      charge_delay_2();
      //
      // initiate conversion
      //
      ADCSRA |= (1 << ADSC);
      //
      // wait for completion
      //
      while (ADCSRA & (1 << ADSC))
         ;
      //
      // save result
      //
      down=ADC;
   

	return ( (up + (1023 - down))/2);
}

 int16_t measure_interval_3 () { 
 
    uint16_t up, down;
	
	  //
      // settle, charge, and wait 3
      //
      settle_delay();
      set(charge_port, charge_pin);
      charge_delay_3();
      //
      // initiate conversion
      //
      ADCSRA |= (1 << ADSC);
      //
      // wait for completion
      //
      while (ADCSRA & (1 << ADSC))
         ;
      //
      // save result
      //
      up = ADC;
      //
      // settle, discharge, and wait 3
      //
      settle_delay();
      clear(charge_port, charge_pin);
      charge_delay_3();
      //
      // initiate conversion
      //
      ADCSRA |= (1 << ADSC);
      //
      // wait for completion
      //
      while (ADCSRA & (1 << ADSC))
         ;
      //
      // save result
      //
      down=ADC;

	return ( (up + (1023 - down))/2);
}


void send_result () {

		
	  //  highByte  = (unsigned char)((command & 0xff00) >> 8); 
	  //  lowerByte = (unsigned char)(command & 0x00ff); 
	 

    put_char (&serial_port, serial_pin_out, (unsigned char)((adc0_avg1 & 0xff00) >> 8)); 
    put_char (&serial_port, serial_pin_out, (unsigned char)(adc0_avg1 & 0x00ff)); 
	//
	put_char (&serial_port, serial_pin_out, (unsigned char)((adc0_avg2 & 0xff00) >> 8)); 
    put_char (&serial_port, serial_pin_out, (unsigned char)(adc0_avg2 & 0x00ff)); 
	//
	put_char (&serial_port, serial_pin_out, (unsigned char)((adc0_avg3 & 0xff00) >> 8)); 
    put_char (&serial_port, serial_pin_out, (unsigned char)(adc0_avg3 & 0x00ff)); 
	//
	put_char (&serial_port, serial_pin_out, (unsigned char)((adc1_avg1 & 0xff00) >> 8)); 
    put_char (&serial_port, serial_pin_out, (unsigned char)(adc1_avg1 & 0x00ff)); 
	//
	put_char (&serial_port, serial_pin_out, (unsigned char)((adc1_avg2 & 0xff00) >> 8)); 
    put_char (&serial_port, serial_pin_out, (unsigned char)(adc1_avg2 & 0x00ff)); 
	//
	put_char (&serial_port, serial_pin_out, (unsigned char)((adc1_avg3 & 0xff00) >> 8)); 
    put_char (&serial_port, serial_pin_out, (unsigned char)(adc1_avg3 & 0x00ff)); 


}

void send_framing () {
      //
	  // for debugging > framing:
      
      put_char(&serial_port, serial_pin_out, 1);
      char_delay();
      put_char(&serial_port, serial_pin_out, 2);
      char_delay();
      put_char(&serial_port, serial_pin_out, 3);
      char_delay();
      put_char(&serial_port, serial_pin_out, 4);
	  
}